import UIKit

struct Vector: ExpressibleByArrayLiteral, CustomStringConvertible, Equatable {
  let x: Int
  let y: Int
  let z: Int
  
  var description: String {
    return "(\(x), \(y), \(z))"
  }
  
  init(_ x: Int, _ y: Int, _ z: Int) {
    self.x = x
    self.y = y
    self.z = z
  }
  
  init(arrayLiteral: Int...) {
    assert(arrayLiteral.count == 3, "Must initialize vector with 3 values.")
    self.x = arrayLiteral[0]
    self.y = arrayLiteral[1]
    self.z = arrayLiteral[2]
  }
  
  static func +(left: Vector, right: Vector) -> Vector {
    return [left.x + right.x, left.y + right.y, left.z + right.z]
  }
  
  static prefix func -(vector: Vector) -> Vector {
    return [-vector.x, -vector.y, -vector.z]
  }
  
  static func -(left: Vector, right: Vector) -> Vector {
    return left + -right
  }
  
  static func *(left: Int, right: Vector) -> Vector {
    return [right.x * left, right.y * left, right.z * left]
  }
  
  static func *(left: Vector, right: Int) -> Vector {
    return right * left
  }
  
  static func *(left: Vector, right: Vector) -> Vector {
    return [left.y * right.z - left.z * right.y, left.z * right.x - left.x * right.z, left.x * right.y - left.y * right.x]
  }
  
  static func ==(left: Vector, right: Vector) -> Bool {
    return left.x == right.x && left.y == right.y && left.z == right.z
  }
  
  static func •(left: Vector, right: Vector) -> Int {
    return left.x * right.x + left.y * right.y + left.z * right.z
  }
}

precedencegroup DotProductPrecedence {
  lowerThan: AdditionPrecedence
  associativity: left
}

infix operator •: DotProductPrecedence

let vectorA: Vector! = [1, 3, 2]
let vectorB: Vector! = [-2, 5, 1]

vectorA + vectorB // (-1, 8, 3)

vectorA - vectorB // (3, -2, 1)

vectorA * 2 * vectorB // (-14, -10, 22)

vectorA == vectorB // false

vectorA • vectorB // 15

vectorA • vectorB + vectorA // 29
